package de.fhdw.wtf.tooling.editor;

import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.DocumentCommand;
import org.eclipse.jface.text.IAutoEditStrategy;
import org.eclipse.jface.text.IDocument;

public class WTFAutoClosingBraceAssist implements IAutoEditStrategy {
	
	@Override
	public void customizeDocumentCommand(final IDocument document, final DocumentCommand command) {
		if (command.text.equals("{")) {
			command.text = "{}";
			this.configureCommand(command);
		}
		if (command.text.equals("[")) {
			command.text = "[]";
			this.configureCommand(command);
		}
		if (command.text.equals("\r\n") || command.text.equals("\n")) {
			int line;
			try {
				line = document.getLineOfOffset(command.offset);
				final int indent = getIndentOfLine(document, line).length();
				
				if (document.getChar(command.offset - 1) == '{' || document.getChar(command.offset - 1) == '[') {
					command.text = "\n" + "\t" + this.produceIndent(indent);
					command.caretOffset = command.offset + 2 + indent;
				} else {
					command.text = "\n" + this.produceIndent(indent);
					command.caretOffset = command.offset + 1 + indent;
				}
				command.shiftsCaret = false;
			} catch (final BadLocationException e) {
				// nothing to do
			}
		}
	}
	
	private String produceIndent(final int indent) {
		final StringBuilder result = new StringBuilder();
		for (int i = 0; i < indent; i++) {
			result.append("\t");
		}
		return result.toString();
	}
	
	private void configureCommand(final DocumentCommand command) {
		
		command.caretOffset = command.offset + 1;
		command.shiftsCaret = false;
	}
	
	public static int findEndOfWhiteSpace(final IDocument document, final int offset, final int end)
			throws BadLocationException {
		int pos = offset;
		while (pos < end) {
			final char c = document.getChar(pos);
			if (c != ' ' && c != '\t') {
				return pos;
			}
			pos++;
		}
		return end;
	}
	
	public static String getIndentOfLine(final IDocument document, final int line) throws BadLocationException {
		if (line > -1) {
			final int start = document.getLineOffset(line);
			final int end = start + document.getLineLength(line) - 1;
			final int whiteend = findEndOfWhiteSpace(document, start, end);
			return document.get(start, whiteend - start);
		} else {
			return " ";
		}
	}
	
}
