package de.fhdw.wtf.tooling.editor.proposals;

import java.util.Vector;

import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.ITextViewer;

public class AttributeModifierProposal extends ContextProposal {
	private static AttributeModifierProposal instance;
	
	public static synchronized AttributeModifierProposal create() {
		if (instance == null) {
			instance = new AttributeModifierProposal();
		}
		return instance;
	}
	
	public final static Vector<String> PREDECESSORS = new Vector<>();
	
	private AttributeModifierProposal() {
		super(new String[] { "prior", "transient", "findable", "mutable", "optional", "symmetric(<target>)" });
		PREDECESSORS.add("String");
		PREDECESSORS.add("Integer");
	}
	
	@Override
	protected Integer skipToToken(final ITextViewer viewer, final int offset) {
		return offset;
	}
	
	@Override
	protected boolean isValid(final char c, final int offset, final IDocument doc) {
		return this.checkForTypeSuffix(doc, offset);
	}
	
	@Override
	protected boolean isInvalid(final char c, final int offset, final IDocument doc) {
		return c == '}' || c == '[' || c == ':' || c == '{' || c == ';' || offset == 0;
	}
	
	private boolean checkForTypeSuffix(final IDocument doc, final int offset) {
		
		int offsetCopy = offset;
		
		try {
			char c = doc.getChar(offsetCopy);
			if (Character.isWhitespace(c)) {
				final StringBuilder builder = new StringBuilder();
				offsetCopy--;
				if (offsetCopy < 0) {
					return false;
				}
				while (true) {
					c = doc.getChar(offsetCopy);
					if (Character.isWhitespace(c) || offsetCopy == 0) {
						break;
					}
					builder.append(c);
					offsetCopy--;
				}
				final String currentString = builder.reverse().toString();
				for (final String s : PREDECESSORS) {
					if (s.equals(currentString)) {
						return true;
					}
				}
				return false;
			}
		} catch (final BadLocationException e) {
			return false;
		}
		return false;
	}
	
	public void generateStandardPredecessors() {
		PREDECESSORS.clear();
		PREDECESSORS.add("String");
		PREDECESSORS.add("Integer");
	}
}
